<?php

namespace Drupal\commerce_stock_notifications\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class AdminConfigForm.
 */
class AdminConfigForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'commerce_stock_notifications_admin_config_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'commerce_stock_notifications.adminconfig',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = $this->getDefaultValues();

    $form['commerce_stock_notifications'] = [
      '#type' => 'details',
      '#title' => $this->t('Stock Notification Settings'),
      '#open' => TRUE,
    ];

    $form['commerce_stock_notifications']['oos_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Out of Stock Message'),
      '#description' => $this->t('A message that the users will see when the product is out of stock.'),
      '#required' => TRUE,
      '#default_value' => $config['oos_message'],
    ];

    $form['commerce_stock_notifications']['success_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Success Message'),
      '#description' => $this->t('A message that the users will see when they successfully submit their email for notifications.'),
      '#required' => TRUE,
      '#default_value' => $config['success_message'],
    ];

    $form['commerce_stock_notifications']['duplicate_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Duplicate Message'),
      '#description' => $this->t('A message that the users will see when they submit a duplicate email address for notifications.'),
      '#required' => TRUE,
      '#default_value' => $config['duplicate_message'],
    ];

    $form['commerce_stock_notifications']['purge_interval'] = [
      '#type' => 'number',
      '#title' => $this->t('Purge Interval'),
      '#description' => $this->t('Sent notifications will be kept in the commerce_stock_notifications table for X days.'),
      '#field_suffix' => $this->t('days'),
      '#size' => 3,
      '#required' => TRUE,
      '#default_value' => intval($config['purge_interval'], 10),
      '#attributes' => [
        'min' => 1,
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    $days = $form_state->getValue('purge_interval');
    if (!is_numeric($days)) {
      $form_state->setErrorByName('purge_interval', $this->t('Purge interval must be a numeric value.'));
    }
    elseif (intval($days, 10) <= 0) {
      $form_state->setErrorByName('purge_interval', $this->t('Purge interval must be a positive number.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $form_state->cleanValues();
    $config = $form_state->getValues();

    $this->config('commerce_stock_notifications.adminconfig')
      ->setData($config)
      ->save();
  }

  /**
   * Returns the configuration values.
   *
   * @return array
   *   Associative array containing the configuration.
   */
  private function getDefaultValues() {
    // Fetch config.
    $config = $this->config('commerce_stock_notifications.adminconfig')
      ->getRawData();
    $defaults = [];
    // OOS Message.
    $defaults['oos_message'] = isset($config['oos_message']) ? $config['oos_message'] : 'Please notify me when this product is back in stock.';
    $defaults['success_message'] = isset($config['success_message']) ? $config['success_message'] : 'You will be notified when this product is back in stock.';
    $defaults['duplicate_message'] = isset($config['duplicate_message']) ? $config['duplicate_message'] : 'A request for notification has already been made for this product with this email address.';
    $defaults['purge_interval'] = isset($config['purge_interval']) ? $config['purge_interval'] : '30';
    return $defaults;
  }

}
