<?php

namespace Drupal\commerce_stock_notifications\Plugin\QueueWorker;

use Drupal\commerce_product\Entity\ProductVariation;
use Drupal\Core\Queue\QueueWorkerBase;

/**
 * Sends out notifications when items are in stock.
 *
 * @QueueWorker(
 *   id = "commerce_stock_notifications",
 *   title = @Translation("Commerce Stock Notification Queue"),
 *   cron = {"time" = 120}
 * )
 */
class CommerceStockNotifyQueue extends QueueWorkerBase {

  /**
   * Mail service for sending out the mail.
   *
   * @var mixed
   */
  private $mailManager;

  /**
   * CommerceStockNotifyQueue constructor.
   *
   * @param array $configuration
   *   Configuration.
   * @param mixed $plugin_id
   *   Plugin ID.
   * @param mixed $plugin_definition
   *   Plugin Definition.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->mailManager = \Drupal::service('plugin.manager.mail');
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    $transaction = \Drupal::database()->startTransaction();

    try {
      $result = \Drupal::database()
        ->select('commerce_stock_notifications', 'csn')
        ->fields('csn', ['notification_id', 'sent_time'])
        ->condition('notification_id', $data->notification_id, '=')
        ->isNull('sent_time')
        ->execute();

      $result->allowRowCount = TRUE;

      $num_of_results = $result->rowCount();

      if ($num_of_results != 0) {
        $user = user_load_by_mail($data->email);
        $product = ProductVariation::load($data->product_id);
        // Parameters.
        $params = [
          'mail_title' => '"' . $product->getTitle() . '" is now in stock!',
          'message' => 'Hello, ' . $user->getDisplayName() . '. We just want to notify you that the item "' . $product->getTitle() . '" is now in stock!',
        ];
        // Send a message.
        $this->mailManager->mail('commerce_stock_notifications', 'notify', $data->email, $user->getPreferredLangcode(), $params, NULL, TRUE);

        \Drupal::database()
          ->update('commerce_stock_notifications')
          ->fields(['sent_time' => time()])
          ->condition('notification_id', $data->notification_id, '=')
          ->execute();
      }
    } catch (\Exception $e) {
      $transaction->rollBack();
      return;
    }
  }

}
